require 'erb'
require 'yaml'
require 'ostruct'
require 'rake'
require 'awesome_print'
require 'pry'
require 'logger'

Rake.application.options.trace = true
ENV['EDITOR'] = 'vim' if ENV['EDITOR'].nil?
ENV['SHELL'] = '/bin/bash' if ENV['SHELL'].nil?

module SigtermExtensions
  def self.root_path(*args)
    File.join File.expand_path('../', __dir__), *args
  end

  def self.ext_path(*args)
    File.join SigtermExtensions.root_path, 'lib/sigterm_extensions', args
  end

  def self.revision
    `git log --pretty=format:%h --abbrev=11 -n 1`
  end

  alias :orig_require :require
  def self.require s
    print "Requires #{s}\n" if orig_require(s)
  end

  def self.get_classes_from mod
    mod.constants.select { |k| mod.const_get(k).instance_of? Class }
  end

  autoload :BacktraceCleaner, SigtermExtensions.ext_path('backtrace_cleaner')
  autoload :Callbacks, SigtermExtensions.ext_path('callbacks')
  autoload :Concern, SigtermExtensions.ext_path('concern')
  autoload :Configurable, SigtermExtensions.ext_path('configurable')
  autoload :DescendantsTracker, SigtermExtensions.ext_path('descendants_tracker')
  autoload :GemMethods, SigtermExtensions.ext_path('gem_methods')
  autoload :Git, SigtermExtensions.root_path('lib/git')
  autoload :HashBinding, SigtermExtensions.ext_path('hash_binding')
  autoload :Inflector, SigtermExtensions.ext_path('inflector')
  autoload :Lazy, SigtermExtensions.ext_path('lazy')
  autoload :Paths, SigtermExtensions.ext_path('paths')
  autoload :ProxyObject, SigtermExtensions.ext_path('proxy_object')
  autoload :Wrappable, SigtermExtensions.ext_path('wrappable')

  def self.all_requirements
    $LOADED_FEATURES.
      select { |feature| feature.include? 'gems' }.
      map { |feature| File.dirname(feature) }.
      map { |feature| feature.split('/').last }.
      uniq.sort
  end

  def self.all_required_unfiltered
    $LOADED_FEATURES
  end

  class RuntimeClassBase
    attr_accessor :settings
  end

  class RuntimeClass
    def self.new(base=RuntimeClassBase, options={}, &block)
      base = Class.new(base)
      base.class_eval(&block) if block_given?
      base
    end
  end
end

require_relative 'sigterm_extensions/all'

